''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' NAME:		check_msmq.vbs
' VERSION:	0.2
' AUTHOR:	Alexander Rudolf (alexander.rudolf@saxsys.de)
'
' COMMENT:	Script for Checking MSMQ metrics
'		for use with Nagios and NSClient++
'
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Option explicit
Dim strQueueName
Dim objArgs
Dim objMsmqApp
Dim objMsmqMgmt
Dim arrQueues
Dim valMessageCount
Dim valBytesInQueue
Dim valMaxWarn
Dim valMaxCrit
Dim i

Set objArgs = WScript.Arguments

If objArgs.Count = 0 Then
	Wscript.StdOut.WriteLine "CRITICAL - Missing arguments"
	WScript.Quit(2)
End if

if objArgs(0) = "--help" Then
	Wscript.StdOut.WriteLine ""
	Wscript.StdOut.WriteLine "CHECK_MSMQ Usage"
	Wscript.StdOut.WriteLine ""
	Wscript.StdOut.WriteLine "check_msmq.vbs [function] [options]"
	Wscript.StdOut.WriteLine ""
	Wscript.StdOut.WriteLine "Functions:"
	Wscript.StdOut.WriteLine vbTab & "ListQueueNames"
	Wscript.StdOut.WriteLine vbTab & "MessageCount [QueueName] [warn] [crit]"
	Wscript.StdOut.WriteLine vbTab & "BytesInQueue [QueueName] [warn] [crit]"
	Wscript.StdOut.WriteLine ""
	Wscript.StdOut.WriteLine ""
	Wscript.StdOut.WriteLine ""
	WScript.Quit(0)
End if

set objMsmqApp = CreateObject("MSMQ.MSMQApplication")
arrQueues = objMsmqApp.ActiveQueues

Select Case objArgs(0)
	Case "ListQueueNames"
		For i = LBound(arrQueues) To UBound(arrQueues)
			set objMsmqMgmt = CreateObject("MSMQ.MSMQManagement")
			strQueueName = arrQueues(i)
			wscript.echo strQueueName
		Next
		WScript.Quit(0)

	Case "OutMessageCount"
		Wscript.StdOut.WriteLine "CRITICAL - Argument 'OutMessageCount' no longer supported - try MessageCount" & objArgs(0)
		WScript.Quit(2)

	Case "MessageCount"
		If objArgs.Count <> 4 Then
			Wscript.StdOut.WriteLine "CRITICAL - Bad number of arguments for " & objArgs(0)
			WScript.Quit(2)
		End if
			valMaxWarn = CInt(objArgs(2))
			valMaxCrit = CInt(objArgs(3))
		If valMaxCrit < valMaxWarn Then
			Wscript.echo "CRITICAL - critical below warning level"
			Wscript.quit(2)
		End If
		For i = LBound(arrQueues) To UBound(arrQueues)
			set objMsmqMgmt = CreateObject("MSMQ.MSMQManagement")
			strQueueName = arrQueues(i)
			if ucase(strQueueName) = ucase(objArgs(1)) Then
				objMsmqMgmt.Init ,,strQueueName
				valMessageCount = objMsmqMgmt.MessageCount
				strQueueName = Replace(strQueueName,"\","\\",1,-1,vbTextCompare)
				If valMessageCount >= valMaxCrit Then
					Wscript.echo "MessageCount CRITICAL - " & strQueueName & ": " & valMessageCount
					Wscript.echo "|MessageCount=" & valMessageCount & ";" & valMaxWarn & ";" & valMaxCrit
					Wscript.quit(2)
				End If
				If valMessageCount >= valMaxWarn Then
					Wscript.echo "MessageCount WARNING - " & strQueueName & ": " & valMessageCount
					Wscript.echo "|MessageCount=" & valMessageCount & ";" & valMaxWarn & ";" & valMaxCrit
					Wscript.quit(1)
				End If
				Wscript.echo "MessageCount OK - " & strQueueName & ": " & valMessageCount
				Wscript.echo "|MessageCount=" & valMessageCount & ";" & valMaxWarn & ";" & valMaxCrit
				Wscript.quit(0)
			End If
		Next
			Wscript.StdOut.WriteLine "CRITICAL - Queue '" & strQueueName & "' not found " & objArgs(1)
			WScript.Quit(2)

	Case "BytesInQueue"
		If objArgs.Count <> 4 Then
			Wscript.StdOut.WriteLine "CRITICAL - Bad number of arguments for " & objArgs(0)
			WScript.Quit(2)
		End if
			valMaxWarn = CInt(objArgs(2))
			valMaxCrit = CInt(objArgs(3))
		If valMaxCrit < valMaxWarn Then
			Wscript.echo "CRITICAL - critical below warning level"
			Wscript.quit(2)
		End If
		For i = LBound(arrQueues) To UBound(arrQueues)
			set objMsmqMgmt = CreateObject("MSMQ.MSMQManagement")
			strQueueName = arrQueues(i)
			if ucase(strQueueName) = ucase(objArgs(1)) Then
				objMsmqMgmt.Init ,,strQueueName
				valBytesInQueue = cint(objMsmqMgmt.BytesInQueue)
				strQueueName = Replace(strQueueName,"\","\\",1,-1,vbTextCompare)
				If valBytesInQueue >= valMaxCrit Then
					Wscript.echo "BytesInQueue CRITICAL - " & strQueueName & ": " & valBytesInQueue
					Wscript.echo "|BytesInQueue=" & valBytesInQueue & ";" & valMaxWarn & ";" & valMaxCrit
					Wscript.quit(2)
				End If
				If valBytesInQueue >= valMaxWarn Then
					Wscript.echo "BytesInQueue WARNING - " & strQueueName & ": " & valBytesInQueue
					Wscript.echo "|BytesInQueue=" & valBytesInQueue & ";" & valMaxWarn & ";" & valMaxCrit
					Wscript.quit(1)
				End If
				Wscript.echo "BytesInQueue OK - " & strQueueName & ": " & valBytesInQueue
				Wscript.echo "|BytesInQueue=" & valBytesInQueue & ";" & valMaxWarn & ";" & valMaxCrit
				Wscript.quit(0)
			End If
		Next
			Wscript.StdOut.WriteLine "CRITICAL - Queue '" & strQueueName & "' not found " & objArgs(1)
			WScript.Quit(2)

	Case else
		Wscript.echo "CRITICAL - Bad argument"
		Wscript.quit(2)
End Select
